//: /be/optiloading/Genetics.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import java.util.ArrayList;
import java.util.Collections;

/** 
 * Genetics class which stores genetic settings and starts/stops the genetic thread.
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class Genetics {
	// Genetic settings standards (used by the TestCase)
	public static float TARGET = 0f;
	public static float CROSSRATE = 0.7f;
	public static float MUTRATE = 0.01f;
	public static float ELITERATE = 0.02f;
	public static int POPSIZE = 60;
	public static int CREATION = 0;
	public static int SELECTION = 0;
	public static int CROSSOVER = 0;
	public static int MUTATION = 1;
	public static int GENERATIONS = 100;
	public static int RESTARTS = 5;
	public static int OPTIMUM = 25;
	public static boolean TRIM = true;
	public static boolean STRENGTH = true;
	public static boolean STABILITY = true;
	public static GeneticThread genetic;

	/**
	 * start method starts the genetic thread
	 */
	public static void start(){
		setGeneticSettings();
		genetic = new GeneticThread();
		genetic.start();
	}
	
	/**
	 * setGeneticSettings gets the inputed settings from the GUI.
	 */
	public static void setGeneticSettings(){
		Settings settingspanel = Settings.getInstance();
		Gui optiloading = Gui.getInstance();
		TARGET = optiloading.getTarget();
		CROSSRATE = settingspanel.getCrossrate();
		MUTRATE = settingspanel.getMutationrate();
		POPSIZE = settingspanel.getPopulationsize();
		CREATION = settingspanel.getCreation();
		SELECTION = settingspanel.getSelection();
		CROSSOVER = settingspanel.getCrossover();
		MUTATION = settingspanel.getMutation();
		ELITERATE = settingspanel.getEliterate();
		RESTARTS = settingspanel.getRestarts();
		OPTIMUM = settingspanel.getOptimum();
		TRIM = settingspanel.isTrim();
		STRENGTH = settingspanel.isStrength();
		STABILITY = settingspanel.isStability();
		settingspanel = null;
		optiloading = null;
	}
	
	/**
	 * stop method stops the genetic thread
	 */
	public static void stop(){
		genetic.requestStop();
	}
}

// Private class which inherits all Thread methods so as to create a genetic thread and run trough generations of chromosomes.
class GeneticThread extends Thread {
	private static Population pop; //The main population
	private static Population newpop; //Population to add new chromosomes
	private volatile boolean start = true; //Should the thread be started?
	private int restart;
	private ArrayList<Chromosome> bestList = new ArrayList<Chromosome>();
	
	// Overwritten method from the Thread class with runs when the thread is started
	public void run(){		
		while(start == true){ //Run while thread is started
			restart = 0;
			pop = new Population(); // New main population
			pop.createInitialPopulation(); // Add initial population to main population
			Ship usedShip = Ship.getInstance(); // Get a ship object to load the best solution
			Gui optiloading = Gui.getInstance(); // Get a Gui object to update the gui

			// Calculate how many chromosomes to copy from previous population
			int rate = (int) (Genetics.ELITERATE * Genetics.POPSIZE);
			if(rate % 2 != 0){
				rate++;
			}

			Chromosome previousbest = pop.getBestchromosome();
			optiloading.repaintOptiloading(0,pop.getTotalfitness()/Genetics.POPSIZE, previousbest, Ship.getInstance());

			//Run while the thread is started and a local optimum has not been found
			for (int j = 0; ((restart < Genetics.OPTIMUM) && (start==true)); j++){

				Chromosome best = pop.getBestchromosome(); // Get the best Chromosome from the population

				//Check if the previous solution is similar (to later determine if a local optimum has been found)
				if(Math.round(previousbest.getFitness())==Math.round(best.getFitness())){
					restart++;
				} else{restart = 0;}

				if(previousbest.getFitness() != best.getFitness()){
					optiloading.repaintOptiloading(j,pop.getTotalfitness()/Genetics.POPSIZE, best, Ship.getInstance()); //Repaint Gui

					// Make the thread sleep for 50 milliseconds to slow down the update of the gui and so as not to overstress the computer
					try{
						Thread.sleep(50);
					} catch(InterruptedException ex){
						ex.printStackTrace();
					}

				} else {
					optiloading.repaintOptiloading(j,pop.getTotalfitness()/Genetics.POPSIZE); //Repaint Gui
				}

				previousbest = (Chromosome) best.clone();

				// Update log
				optiloading.addLog("Generation = "+String.valueOf(j));
				optiloading.addLog("Mean fitness = "+String.valueOf(pop.getTotalfitness()/Genetics.POPSIZE));
				optiloading.addLog("Best volumes = "+String.valueOf(best.getVolumes()));
				optiloading.addLog("Best fitness = "+String.valueOf(best.getFitness()));
				optiloading.addLog("Best trim = "+String.valueOf(best.getTrim()));
				optiloading.addLog("Best GZ = "+String.valueOf(best.getGM()));
				optiloading.addLog("Best FSC = "+String.valueOf(best.getFSC()));
				optiloading.addLogSeparator();

				// Initialize selected chromosomes
				Chromosome selected1 = null;
				Chromosome selected2 = null;

				// Initialize the new population
				Population newpop = pop.elitism(rate);

				for(int i = 0; i < (Genetics.POPSIZE-rate); i+=2){

					// Select two chromosomes with diferrent fitness using rank or roulette selection
					do{
						switch(Genetics.SELECTION){
							case 0:
							selected1 = pop.selectRankChromosome();
							selected2 = pop.selectRankChromosome();
							break;
							case 1: 
							selected1 = pop.selectRouletteChromosome();
							selected2 = pop.selectRouletteChromosome();
							break;
						}
					} while (selected1.getFitness() == selected2.getFitness());

					// Crossover using the appropriate method
					switch(Genetics.CROSSOVER){
						case 0: selected1.crossOverMean(selected2); break;
						case 1: selected1.crossOverOnePoint(selected2); break;
						case 2: selected1.crossOverTwoPoint(selected2); break;
						case 3: selected1.crossOverUniform(selected2); break;
					}

					// Mutate using the appropriate method
					switch(Genetics.MUTATION){
						case 0: 
						selected1.mutateSwitch();
						selected2.mutateSwitch(); 
						break;
						case 1: 
						selected1.mutateRandom();
						selected2.mutateRandom();
						break;
					}

					// Add to temporary population
					newpop.addChromosome(selected1);
					newpop.addChromosome(selected2);

				}

				pop = newpop; // Assign temporary population to new population
			}
			bestList.add(pop.getBestchromosome());
			
			if(bestList.size()==Genetics.RESTARTS){
				start = false;
				Collections.sort(bestList);
				optiloading.repaintOptiloading(0, 0, bestList.get(Genetics.RESTARTS-1), Ship.getInstance());
				optiloading.stopFinal();
			}
		}
				
	}
	
	// Stops the genetic thread
	public void requestStop(){
		start = false;
	}
}

