//: /be/optiloading/Tank.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import java.util.ArrayList;
import java.io.*;

/** 
 * Tank class used to stow information about the tank and calculate its center of gravity.
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class Tank implements Serializable{
	private float maxvolume;
	private String tankid;
	private float specificgravity;
	private float cargo;
	private float volume;
	private float lcg;
	private float kg;
	private float fsm;
	private float length;
	private int beginframe;
	private int endframe;
	
	private ArrayList<TankData> data = new ArrayList<TankData>(); //ArrayList of TankData objects
	
	/**
	 * Constructor used to create a new tank.
	 * @param id String to identify the tank.
	 * @param max Floating point value indicating the maximum value.
	 */
	public Tank(String id, float max, float l, int bf, int ef){
		tankid = id;
		maxvolume = max;
		length = l;
		beginframe = bf;
		endframe = ef;
	}
	
	/**
	 * Method for adding sounding data to the tank.
	 * @param vol Floating point value indicating a volume.
	 * @param flo Floating point value that indicated the longitudinal center of gravity corresponding with the indicated volume.
	 */
	public void addData(float vol, float flo, float kge, float fsmo){
		data.add(new TankData(vol,flo,kge,fsmo)); // New TankData object added to the ArrayList
	}
	
	/**
	 * Method to set the volume of a tank and calculate the longitudinal center of gravity.
	 * @param vol Floating point value indicating the volume to be set.
	 * @param specg Floating point value indicating the specific gravity of the loaded volume
	 */
	public void setVolume(float vol, float specg){
		volume = vol; // Set volume
		specificgravity = specg; // Set specific gravity
		cargo = (vol)*(specg); // Set weight
		for(int i = 0; i < data.size(); ++i) { // Run trough the tank data to find the longitudinal center of gravity
			TankData x = data.get(i);
			if(volume < x.volume) { // If the volume of the object is greater then we've gone past the desired volume
				TankData y = data.get(i-1); // Get the previous object to interpolate
				lcg = interpolate(volume,y.volume,x.volume,y.lcg,x.lcg);
				kg = interpolate(volume,y.volume,x.volume,y.kg,x.kg);
				fsm = interpolate(volume,y.volume,x.volume,y.fsm,x.fsm);
				break;
			} else if(volume == x.volume) { // The desired volume is equal to the objects volume so we found the object
				lcg = x.lcg;
				kg = x.kg;
				fsm = x.fsm;
				break;
			}
		}
	}
	
	/**
	 * Method to get the Longitudinal Center of Gravity (LCG) of the tank
	 * @return Floating point value containing the LCG of the tank
	 */
	public float getLCG(){
		return lcg;
	}
	
	/**
	 * Method to get the KG of the tank
	 * @return Floating point value containing the KG of the tank
	 */
	public float getKG(){
		return kg;
	}
	
	/**
	 * Method to get the Longitudinal Free Surface Moment (FSM) of the tank
	 * @return Floating point value containing the FSM of the tank
	 */
	public float getFSM(){
		return fsm;
	}
	
	/**
	 * Method to get the weight of the cargo in the tank
	 * @return Floating point value containing the weight of the cargo in the tank
	 */
	public float getCargo(){
		return cargo;
	}
	
	/**
	 * Method to get the volume of the cargo in the tank
	 * @return Floating point value containing the volume of the cargo in the tank
	 */
	public float getVolume(){
		return volume;
	}
	
	/**
	 * Method to get the tank's identification
	 * @return String value containing the tank's identification
	 */
	public String getTankid(){
		return tankid;
	}
	
	/**
	 * Method to get the maximum volume the tank
	 * @return Floating point value containing the maximum volume the tank
	 */
	public float getMaxvolume(){
		return maxvolume;
	}
	
	/**
	 * Method to get the length of the tank
	 * @return Floating point value containing the length of the tank
	 */
	public float getLength(){
		return length;
	}
	
	/**
	 * Method to get the begin frame of the tank
	 * @return Floating point value containing the begin frame of the tank
	 */
	public int getBeginframe(){
		return beginframe;
	}
	
	/**
	 * Method to get the end frame of the tank
	 * @return Floating point value containing the end frame of the tank
	 */
	public int getEndframe(){
		return endframe;
	}
	
	/**
	 * Method to get the specific gravity of the cargo in the tank
	 * @return Floating point value containing the specific gravity of the cargo in the tank
	 */
	public float getSpecificGravity(){
		return specificgravity;
	}
	
	// Method used to interpolate between value
	// refnum is the number for which we want the value
	// refnum is between refsmall and reflarge which correspond to solsmall and solllargo
	private static float interpolate(float refnum, float refsmall, float reflarge, float solsmall, float sollarge){
		float x = refnum - refsmall;
		float y = reflarge - refsmall;
		float z = sollarge - solsmall;
		float solution = (x/y)*z;
		return (solution + solsmall);
	}
	
	// Inner class used to store tank data, which is set when the addData(float vol, float flo) method of the Tank object is called
	// These objects are stored in the ArrayList data contained within each Tank object
	class TankData implements Serializable{
		float volume;
		float lcg;
		float kg;
		float fsm;

		// Constructor which set the longitudinal center of gravity corresponding with the volume
		TankData(float vol, float flo, float kge, float fsmo){
			volume = vol;
			lcg = flo;
			kg = kge;
			fsm = fsmo;
		}
	}
}